@@ -2,16 +2,20 @@ require "xml_security"
 require "onelogin/ruby-saml/attribute_service"
 require "onelogin/ruby-saml/utils"
 
+# Only supports SAML 2.0
 module OneLogin
   module RubySaml
+
+    # SAML2 Toolkit Settings
+    #
     class Settings
       def initialize(overrides = {})
         config = DEFAULTS.merge(overrides)
         config.each do |k,v|
           acc = "#{k.to_s}=".to_sym
-          if self.respond_to? acc
+          if respond_to? acc
             value = v.is_a?(Hash) ? v.dup : v
-            self.send(acc, value)
+            send(acc, value)
           end
         end
         @attribute_consuming_service = AttributeService.new
@@ -24,6 +28,7 @@ module OneLogin
       attr_accessor :idp_cert
       attr_accessor :idp_cert_fingerprint
       attr_accessor :idp_cert_fingerprint_algorithm
+      attr_accessor :idp_attribute_names
       # SP Data
       attr_accessor :issuer
       attr_accessor :assertion_consumer_service_url
@@ -39,18 +44,22 @@ module OneLogin
       attr_accessor :protocol_binding
       attr_accessor :attributes_index
       attr_accessor :force_authn
-      attr_accessor :security
       attr_accessor :certificate
       attr_accessor :private_key
       attr_accessor :authn_context
       attr_accessor :authn_context_comparison
       attr_accessor :authn_context_decl_ref
       attr_reader :attribute_consuming_service
+      # Work-flow
+      attr_accessor :security
+      attr_accessor :soft
       # Compability
       attr_accessor :assertion_consumer_logout_service_url
       attr_accessor :assertion_consumer_logout_service_binding
 
-      def single_logout_service_url()
+      # @return [String] Single Logout Service URL.
+      #
+      def single_logout_service_url
         val = nil
         if @single_logout_service_url.nil?
           if @assertion_consumer_logout_service_url
@@ -62,12 +71,16 @@ module OneLogin
         val
       end
 
-      # setter
-      def single_logout_service_url=(val)
-        @single_logout_service_url = val
+      # Setter for the Single Logout Service URL.
+      # @param url [String].
+      #
+      def single_logout_service_url=(url)
+        @single_logout_service_url = url
       end
 
-      def single_logout_service_binding()
+      # @return [String] Single Logout Service Binding.
+      #
+      def single_logout_service_binding
         val = nil
         if @single_logout_service_binding.nil?
           if @assertion_consumer_logout_service_binding
@@ -79,27 +92,53 @@ module OneLogin
         val
       end
 
-      # setter
-      def single_logout_service_binding=(val)
-        @single_logout_service_binding = val
+      # Setter for Single Logout Service Binding.
+      # 
+      # (Currently we only support "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect")
+      # @param url [String]
+      #
+      def single_logout_service_binding=(url)
+        @single_logout_service_binding = url
       end
 
+      # Calculates the fingerprint of the IdP x509 certificate.
+      # @return [String] The fingerprint
+      #
+      def get_fingerprint
+        idp_cert_fingerprint || begin
+          idp_cert = get_idp_cert
+          if idp_cert
+            fingerprint_alg = XMLSecurity::BaseDocument.new.algorithm(idp_cert_fingerprint_algorithm).new
+            fingerprint_alg.hexdigest(idp_cert.to_der).upcase.scan(/../).join(":")
+          end
+        end
+      end
+
+      # @return [OpenSSL::X509::Certificate|nil] Build the IdP certificate from the settings (previously format it)
+      #
+      def get_idp_cert
+        return nil if idp_cert.nil? || idp_cert.empty?
+
+        formatted_cert = OneLogin::RubySaml::Utils.format_cert(idp_cert)
+        OpenSSL::X509::Certificate.new(formatted_cert)
+      end
+
+      # @return [OpenSSL::X509::Certificate|nil] Build the SP certificate from the settings (previously format it)
+      #
       def get_sp_cert
-        cert = nil
-        if self.certificate
-          formated_cert = OneLogin::RubySaml::Utils.format_cert(self.certificate)
-          cert = OpenSSL::X509::Certificate.new(formated_cert)
-        end
-        cert
+        return nil if certificate.nil? || certificate.empty?
+
+        formatted_cert = OneLogin::RubySaml::Utils.format_cert(certificate)
+        OpenSSL::X509::Certificate.new(formatted_cert)
       end
 
+      # @return [OpenSSL::PKey::RSA] Build the SP private from the settings (previously format it)
+      #
       def get_sp_key
-        private_key = nil
-        if self.private_key
-          formated_private_key = OneLogin::RubySaml::Utils.format_private_key(self.private_key)
-          private_key = OpenSSL::PKey::RSA.new(formated_private_key)
-        end
-        private_key
+        return nil if private_key.nil? || private_key.empty?
+        
+        formated_private_key = OneLogin::RubySaml::Utils.format_private_key(private_key)
+        OpenSSL::PKey::RSA.new(formated_private_key)
       end
 
       private
@@ -110,10 +149,12 @@ module OneLogin
         :idp_cert_fingerprint_algorithm            => XMLSecurity::Document::SHA1,
         :compress_request                          => true,
         :compress_response                         => true,
+        :soft                                      => true,
         :security                                  => {
           :authn_requests_signed    => false,
           :logout_requests_signed   => false,
           :logout_responses_signed  => false,
+          :want_assertions_signed   => false,
           :metadata_signed          => false,
           :embed_sign               => false,
           :digest_method            => XMLSecurity::Document::SHA1,
